/*
 * $Id: card.h,v 1.4 1998/04/18 20:27:39 wjg Exp $
 *
 * Definitions for card interface
 *
 * COPYRIGHT:
 *	Copyright 1997 Wolf Geldmacher, Comunicon AG, CH-9052 Niederteufen.
 *	All rights reserved. See file COPYING.
 *
 */
#ifndef _card_h
#define _card_h

#ifdef hardcode_os
/*
 * Operating system support
 */
#define __WIN32__	1	/* Windows 32 bit (Win95/NT)		*/
#endif

#ifdef __GNUC__
#define RCSID static char rcs_id[] __attribute__ ((unused)) =
#else
#define RCSID static char rcs_id[] = 
#endif

/*
 * Drive support
 */
#ifndef INCLUDE_ASE
#define INCLUDE_ASE	0	/* for ASE Drive			*/
#endif
#ifndef INCLUDE_GPR
#define INCLUDE_GPR	0	/* for GPR400 Drive			*/
#endif
#ifndef INCLUDE_REFLEX
#define INCLUDE_REFLEX	0	/* for SLB Reflex60 Reader		*/
#endif

/*
 * Global definitions for library
 */
#define CARD_VERSION	"1.0"	/* Version of library			*/
#define CARD_NAME	"card"	/* Name of library			*/
#define CARD_BUFSIZE	256	/* A small buffer size			*/
#define CARD_PMAX	1024	/* Buffer for path names		*/

/*
 * Common includes
 */
#include <stdio.h>		/* sprintf et al.			*/
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>
#include "port.h"		/* portability functions		*/

/*
 * Operating System dependencies
 */
#ifdef __WIN32__
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#undef WIN32_LEAN_AND_MEAN
#define DllImport	__declspec( dllimport )
#define DllExport	__declspec( dllexport )
#else
#define DllImport
#define DllExport
#endif

/*
 * Definition of global constants
 */
#ifndef TRUE
#define TRUE	1
#endif

#ifndef FALSE
#define FALSE	0
#endif

/*
 * Type definitions
 */
typedef struct card_data card_data;
typedef struct card_reader card_reader;

typedef int (*reader_init)(	/* Function to initialize a reader	*/
#ifdef __STDC__
	card_reader *reader,	/* Pointer to reader information	*/
	int port,		/* I: Port reader is connected to	*/
	int timeout		/* I: Timeout for reader commands	*/
#endif
);

typedef int (*reader_on)(	/* Function to turn a reader on		*/
#ifdef __STDC__
	card_reader *reader,	/* Pointer to reader information	*/
	unsigned char *atr,	/* O: Anwer to reset of card		*/
	unsigned int *atrlenp	/* I/O: length of ATR response		*/
#endif
);

typedef int (*reader_off)(	/* Function to turn a reader off	*/
#ifdef __STDC__
	card_reader *reader	/* Pointer to reader information	*/
#endif
);

typedef int (*reader_send)(	/* Function to issue a ISO7816-3 send	*/
#ifdef __STDC__
	card_reader *reader,	/* Pointer to reader information	*/
	unsigned char cla,	/* I: Class byte			*/
	unsigned char ins,	/* I: Instruction byte			*/
	unsigned char p1,	/* I: P1 byte				*/
	unsigned char p2,	/* I: P2 byte				*/
	unsigned short len,	/* I: LEN of data			*/
	unsigned char *data,	/* I: data to send			*/
	unsigned char *ackp,	/* O: Acknowledge byte 			*/
	unsigned char *sw1p,	/* O: Contains SW1 for last call	*/
	unsigned char *sw2p	/* O: Contains SW2 for last call	*/
#endif
);

typedef int (*reader_recv)(	/* Function to issue a ISO7816-3 recv	*/
#ifdef __STDC__
	card_reader *reader,	/* Pointer to reader information	*/
	unsigned char cla,	/* I: Class byte			*/
	unsigned char ins,	/* I: Instruction byte			*/
	unsigned char p1,	/* I: P1 byte				*/
	unsigned char p2,	/* I: P2 byte				*/
	unsigned short len,	/* I: LEN of data			*/
	unsigned char *data,	/* O: data received			*/
	unsigned char *ackp,	/* O: Acknowledge byte 			*/
	unsigned char *sw1p,	/* O: Contains SW1 for last call	*/
	unsigned char *sw2p	/* O: Contains SW2 for last call	*/
#endif
);

typedef char *(*reader_vers)(	/* Function to get reader version	*/
#ifdef __STDC__
	card_reader *reader	/* Pointer to reader information	*/
#endif
);

typedef char *(*reader_stat)(	/* Function to get a reader status	*/
#ifdef __STDC__
	card_reader *reader	/* Pointer to reader information	*/
#endif
);

/*
 * More than one type of reader is supported. Each reader type has an
 * entry of the following structure:
 */
struct card_reader {		/* Card reader description structure	*/
	char *name;		/* Card reader name			*/
	reader_init init;	/* Initialize a card reader		*/
	reader_on on;		/* Turn reader on and reset card	*/
	reader_send send;	/* Send an ISO7816-3 message to a card	*/
	reader_recv recv;	/* Receive a message from a card	*/
	reader_off off;		/* Turn card reader off			*/
	reader_stat stat;	/* Get card reader status		*/
	reader_vers vers;	/* Get card reader version		*/
	int status;		/* Status of reader			*/
};

/*
 * User visible data available throughout the interface
 */
struct card_data {		/* ClientData structure for card	*/
	card_reader *reader;	/* link to reader			*/
	unsigned char cla;	/* command class byte			*/
	unsigned char ins;	/* instruction byte			*/
	unsigned char p1;	/* parameter byte 1			*/
	unsigned char p2;	/* parameter byte 2			*/
	unsigned short len;	/* length of expected return		*/
	unsigned char *data;	/* pointer to data buffer		*/
	unsigned char ack;	/* Acknowledge byte for last command	*/
	unsigned char sw1;	/* Contains SW1 for last call		*/
	unsigned char sw2;	/* Contains SW2 for last call		*/
	char *verbose;		/* Pointer to verbose message		*/
};

#endif
