/*
 * NAME:
 *	gpr400.c -- lowlevel interface to test gpr400 PCMCIA driver
 *
 * DESCRIPTION:
 *	This program test the interface to the GPR400 reader. It is an
 *	interactive program that will prompt for commands and then will
 *	exercise the selected function.
 *
 * COPYRIGHT:
 *	Copyright 1997 Wolf Geldmacher, Comunicon AG, CH-9052 Niederteufen.
 *	All rights reserved. See file COPYING.
 */
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include "gpr400.h"

static char rcs_id[] = "$Id: gpr400.c,v 1.3 2001/03/25 18:56:50 wjg Exp $";

void
do_status(int fd)
{
	struct gpr400_status reader;

	/*
	 * Get reader status
	 */
	if (ioctl(fd, GPR400_STATUS, &reader) < 0) {
		perror("status");
		return;
	}

	printf("status = 0x%02x\n", reader.status);
	printf("os_version = 0x%02x\n", reader.os_version);
	printf("flash_mem = 0x%02x\n", reader.flash_mem);
	printf("manufacturer = 0x%02x\n", reader.manufacturer);
	printf("rom_sum = %d\n", reader.rom_sum);
	printf("ram_sum = %d\n", reader.ram_sum);
	printf("flash_sum = %d\n", reader.flash_sum);
	printf("reg1 = 0x%02x, reg2 = 0x%02x\n", reader.reg1, reader.reg2);
	printf("info = 0x%02x, ins = 0x%02x\n", reader.info,
		reader.card_inserted);

}

void
printhex(unsigned char *data, int len)
{
	int i;

	for (i = 0; i < len; ++i) {
		if (i % 16 == 0)
			printf("%04x\t", i);
		printf("%02x%s", data[i],
			((i+1)%16 ? ((i+1)%8 ? "" : " ") : "\n"));
	}
	printf("\n");
}

int
main()
{
	char *prompt = "(r)eset, (o)pen, clos(e), s(u)spend, (q)uit, (s)tatus, (p)oweroff, se(l)ect";
	char buffer[1024];
	int fd = open("/dev/gpr400", O_RDWR);
	struct gpr400_ram ram;
	struct gpr400_cmd cmd;
	struct gpr400_atr atr;
	struct gpr400_status status;
	int i;

	if (fd < 0) {
		perror("open reader");
		return 1;
	}

	while (puts(prompt),gets(buffer)) {
		switch (buffer[0]) {
		case 'q':		/* quit	*/
			exit(0);

		case 'r':		/* reset reader */
			if (ioctl(fd, GPR400_RESET, 0) < 0)
				perror("reset");
			break;

		case 'o':
			if (ioctl(fd, GPR400_OPEN, &atr) < 0)
				perror("open");
			else
				printhex(atr.data, atr.len);
			break;

		case 'e':
			if (ioctl(fd, GPR400_CLOSE, 0) < 0)
				perror("close");
			break;

		case 'u':
			if (ioctl(fd, GPR400_STNDBY, 0) < 0)
				perror("suspend");
			break;

		case 'p':
			if (ioctl(fd, GPR400_PWROFF, 0) < 0)
				perror("poweroff");
			break;

		case 's':
			do_status(fd);
			break;

		case 'l':
			if (ioctl(fd, GPR400_SELECT, 0) < 0)
				perror("select");
			break;

		case ' ': case '\n':
			break;

		default:
			printf("unknown command\n");
			break;
		}
	}

	/* NOT REACHED */
	close(fd);
	return 0;
}
