/*
 * $Id: status.h,v 1.2 1998/04/14 20:05:41 wjg Exp $
 *
 * NAME:
 *	status.h -- decoding of card status bytes
 *
 * DESCRIPTION:
 *	The functions in this file are used to decode the standardized
 *	parts of a card's status. The mechanism uses tables that can be
 *	extended by card specific items.
 *
 * COPYRIGHT:
 *	Copyright 1997 Wolf Geldmacher, Comunicon AG, CH-9052 Niederteufen.
 *	All rights reserved. See file COPYING.
 *
 */
#ifndef status_h
#define status_h

#define S_TSIZE		512	/* Size of status hash table		*/
#define S_BSIZE		512	/* Size of static status message buffer	*/

#ifdef __cplusplus
extern "C" {
#endif

int				/* 1 for success, 0 for failure		*/
addCardStatus(			/* Dynamically add Card Status Codes	*/
#ifdef __STDC__
	const char *code,	/* 4 byte hex code, '?' is wildcard	*/
	const char *meaning	/* Meaning of code, '?' is argument	*/
#endif
);

int				/* 1 for success, 0 for failure		*/
deleteCardStatus(		/* Dynamically delete Card Status Codes	*/
#ifdef __STDC__
	const char *code	/* 4 byte hex code to delete		*/
#endif
);

char *				/* Ptr to decoded message, 0 for fail	*/
decodeCardStatus(		/* Decode card status bytes		*/
#ifdef __STDC__
	unsigned char sw1,	/* First status byte			*/
	unsigned char sw2,	/* Second status byte			*/
	char *buffer,		/* Buffer for result, 0 = malloc	*/
	unsigned len		/* sizeof(buffer), 0 = static return	*/
#endif
);

char *				/* Ptr to code, 0 for end		*/
listCardStatus(			/* List decoding of status bytes	*/
#ifdef __STDC__
	void **state,		/* space for state variable, init ptr0	*/
	char **meaning		/* ptr to meaning for code		*/
#endif
);

#ifdef __cplusplus
}
#endif

#endif
