/*
 * $Id: table.h,v 1.2 1998/04/14 20:05:41 wjg Exp $
 *
 * NAME:
 *	table.h -- simple hash table
 *
 * DESCRIPTION:
 *	This is the interface to a set of simple hash table functions.
 *
 * COPYRIGHT:
 *	Copyright 1997 Wolf Geldmacher, Comunicon AG, CH-9052 Niederteufen.
 *	All rights reserved. See file COPYING.
 *
 */
#ifndef table_h
#define table_h

/*
 * Type definitions
 */
typedef struct link link_t;	/* Link type for singly linked list	*/
typedef struct table table_t;	/* Table type for hash table		*/
typedef int			/* <0: k<k(d), ==0: k==k(d), >0 k>k(d)	*/
(*cmpf_t)(			/* Pointer to key comparison function	*/
#ifdef __STDC__
	const void *k,		/* Key to compare			*/
	const void *d		/* data item in table to compare to	*/
#endif
);
typedef unsigned		/* hash value				*/
(*hf_t)(			/* Pointer to hash function		*/
#ifdef __STDC__
	const void *key		/* Pointer to key to generate hash of	*/
#endif
);
typedef void
(*ff_t)(			/* Pointer to cleanup function		*/
#ifdef __STDC__
	void *data		/* Pointer to item to cleanup		*/
#endif
);

/*
 * Structures
 */
struct link {			/* A link in a singly linked list	*/
	link_t *next;		/* pointer to next link			*/
	void *data;		/* pointer to data			*/
};

struct table {			/* A hash table				*/
	cmpf_t compare;		/* Comparison function used in table	*/
	hf_t hash;		/* Hash function used in table		*/
	unsigned size;		/* size of table			*/
	link_t *base[1];	/* base array of table			*/
				/* ... extends past end			*/
};

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Prototypes
 */
link_t *			/* Pointer to new link			*/
newLink(			/* Create/initialize a new link		*/
#ifdef __STDC__
	link_t *next,		/* pointer to next of new link		*/
	void *data		/* pointer to data of new link		*/
#endif
);

void *				/* Pointer to deleted link's data	*/
deleteLink(			/* Delete an existing link		*/
#ifdef __STDC__
	link_t **goner		/* Pointer to link to delete/next link	*/
#endif
);

table_t *			/* Pointer to new table			*/
newTable(			/* Create/initialize a new table	*/
#ifdef __STDC__
	unsigned size,		/* Size of base array			*/
	cmpf_t compare,		/* Pointer to comparison function	*/
	hf_t hash		/* Pointer to hash function		*/
#endif
);

void
deleteTable(			/* Delete an existing table		*/
#ifdef __STDC__
	table_t **goner,	/* Pointer to table to delete		*/
	ff_t cleanup		/* Pointer to cleanup function or NULL	*/
#endif
);

void *				/* Pointer to item deleted		*/
deleteEntry(			/* Delete an entry in a table		*/
#ifdef __STDC__
	table_t *tp,		/* Pointer to table to delete entry in	*/
	const void *k		/* key to delete			*/
#endif
);

void *				/* Pointer to item found or NULL	*/
lookupTable(			/* Lookup an item in a table		*/
#ifdef __STDC__
	table_t *t,		/* Table to perform lookup in		*/
	const void *k		/* key to lookup			*/
#endif
);

const void *			/* Pointer to item inserted or NULL	*/
insertTable(			/* Insert an item into a table		*/
#ifdef __STDC__
	table_t *t,		/* Table to perform lookup in		*/
	const void *k,		/* key to lookup			*/
	void *data		/* data to insert under this key	*/
#endif
);

const void *			/* Pointer to item or NULL		*/
traverseTable(			/* traverse table one entry at a time	*/
#ifdef __STDC__
	table_t *t,		/* Table to traversal in		*/
	void **state		/* state record, initially &null ptr	*/
#endif
);

#ifdef __cplusplus
}
#endif

#endif
